@@ -4,7 +4,7 @@ from random import sample
 import torch
 import torch.nn as nn
 from astrape.utilities.utils_lightning import set_default_parameters
-from astrape.utilities.utils import rearrange_dims
+from astrape.utilities.utils import rearrange_dims, conv_dim
 from astrape.base.model_base import BaseNet, ConvNet, SegNet
 from typing import Union, List, Dict, Any, Optional, cast, Tuple
 from astrape.exceptions.exceptions import *
@@ -12,7 +12,7 @@ from astrape.models.model_buildingblocks import Down, DoubleConv, Up
 import math
 import numpy as np
 from astrape.constants import *
-
+from torchvision import models
 
 class _MLP(nn.Module):
     r"""Multilayer perceptron with identical # of hidden units.
@@ -56,10 +56,12 @@ class _MLP(nn.Module):
         batch_size : int,
         optimizer_type : str,
         dropout_p : float,
+        scheduler,
         bn : bool,
         lr : float,
         weight_decay : float,
         l1_strength : float,
+        momentum : float=0.9,
         activation_type : str = "relu", # "relu" or "leaky_relu"
         negative_slope : Optional[float] = None, # negative slope for leaky relu   
         sample_weight : Optional[Union["np.ndarray", "torch.Tensor"]] = None
@@ -83,6 +85,8 @@ class _MLP(nn.Module):
         self.dropout_p = dropout_p
         self.bn = bn
         self.lr = lr
+        self.scheduler = scheduler
+        self.momentum = momentum
         self.weight_decay = weight_decay 
         self.sample_weight = sample_weight
         self.l1_strength = l1_strength
@@ -214,7 +218,9 @@ class MLP(BaseNet):
             'weight_decay',
             'l1_strength',
             'activation_type',
-            'negative_slope'
+            'negative_slope',
+            'scheduler',
+            'momentum'
             ]
         default_parameter_dict = {
             'n_layers' : 2,
@@ -276,7 +282,9 @@ class _ContractingMLP(nn.Module):
         lr : float,
         weight_decay : float,
         l1_strength : float,
+        scheduler,
         activation_type : str = "relu",
+        momentum : float=0.9,
         negative_slope : Optional[float] = None,
         sample_weight : Optional[Union["torch.Tensor", "np.ndarray"]] = None
     ) -> None:
@@ -331,8 +339,9 @@ class _ContractingMLP(nn.Module):
         self.current_units = self.dims
         self.activation_type = activation_type
         self.negative_slope = negative_slope
-        
-        
+        self.scheduler = scheduler
+        self.momentum = momentum
+
         layers = []
         if self.n_classes:
             if int(self.current_units * self.contraction_ratio) <= self.n_classes:
@@ -448,8 +457,6 @@ class _ContractingMLP(nn.Module):
                 fc3 = nn.Linear(self.current_units, 1)
                 layers.append(fc3)
             
-
-            layers.append(nn.Sigmoid())
             self.layers = nn.Sequential(*layers)
 
 
@@ -516,7 +523,9 @@ class ContractingMLP(BaseNet):
             'weight_decay',
             'l1_strength',
             'activation_type',
-            'negative_slope'
+            'negative_slope',
+            'scheduler',
+            'momentum'
             ]
         default_parameter_dict = {
             'contraction_ratio' : 0.5
@@ -550,45 +559,43 @@ class ContractingMLP(BaseNet):
     def forward(self, X):
         return self.model.forward(X)
 
-########################################################## _ContractingMLP & ContractingMLP (Ends) ##########################################################
 
-########################################################## _CustomMLP & CustomMLP (Begins) ##########################################################
-"""
-_CustomMLP and CustomMLP defines an MLP with number of hidden units for hidden layers 
-customized for all layers. 
-
-We can define n_hidden_units for all hidden layers with a list of integers.
-
-The hyperparameters that are only in CustomMLP are 'layer_{n}_dim's (for all hidden layers). 
-
-
-_CustomMLP and CustomMLP are basically identical in structure. The only difference is that
-_CustomMLP is an offspring of nn.Module and CustomMLP is an offspring of BaseNet.
-
-Arguments:
-    - dims (Union[int, Tuple[int]]): The dimension of the data. e.g., 4, (8,8,1)
-
-    - n_classes (Optional[int]) : Number of classes (labels) in the classification task.
-                                    Do not specify this argument (set is as None) when you 
-                                re performing regression tasks.
-    
-    - n_hidden_units_list : Optional[List[int]] : List of all n_hidden_units for all hidden layers.
-                                                  If not specified, it would be an old-school perceptron.
-
-    - batch_size (int) : Batch size. Default : 64 (see utils_lightning.py)
-
-    - optimizer_type (str) : Type of the optimizer. ("adam" or "sgd") Default : "adam" (see utils_lightning.py) 
-                            #TODO : implement other optimizers.(in model_base.py)
-
-    - dropout_p (0<= float < 1) : Dropout probability Default : 0 (see utils_lightning.py)
-
-    - bn (bool) : Whether to use batch normalization or not. Default : True (see utils_lightning.py)
-
-    - weight_decay (0<= float) : coefficient for L2-regularization. Default : 0 (see utils_lightning.py)
-
-    - lr ( 0 < float) : Learning rate. Default: 1e-4 (see utils_lightning.py)
-"""
 class _CustomMLP(nn.Module):
+    r"""
+    _CustomMLP and CustomMLP defines an MLP with number of hidden units for hidden layers 
+    customized for all layers. 
+
+    We can define n_hidden_units for all hidden layers with a list of integers.
+
+    The hyperparameters that are only in CustomMLP are 'layer_{n}_dim's (for all hidden layers). 
+
+
+    _CustomMLP and CustomMLP are basically identical in structure. The only difference is that
+    _CustomMLP is an offspring of nn.Module and CustomMLP is an offspring of BaseNet.
+
+    Arguments:
+        - dims (Union[int, Tuple[int]]): The dimension of the data. e.g., 4, (8,8,1)
+
+        - n_classes (Optional[int]) : Number of classes (labels) in the classification task.
+                                        Do not specify this argument (set is as None) when you 
+                                    re performing regression tasks.
+        
+        - n_hidden_units_list : Optional[List[int]] : List of all n_hidden_units for all hidden layers.
+                                                    If not specified, it would be an old-school perceptron.
+
+        - batch_size (int) : Batch size. Default : 64 (see utils_lightning.py)
+
+        - optimizer_type (str) : Type of the optimizer. ("adam" or "sgd") Default : "adam" (see utils_lightning.py) 
+                                #TODO : implement other optimizers.(in model_base.py)
+
+        - dropout_p (0<= float < 1) : Dropout probability Default : 0 (see utils_lightning.py)
+
+        - bn (bool) : Whether to use batch normalization or not. Default : True (see utils_lightning.py)
+
+        - weight_decay (0<= float) : coefficient for L2-regularization. Default : 0 (see utils_lightning.py)
+
+        - lr ( 0 < float) : Learning rate. Default: 1e-4 (see utils_lightning.py)
+    """
     def __init__(
         self,
         dims : Union[int, Tuple[int]],
@@ -600,9 +607,11 @@ class _CustomMLP(nn.Module):
         lr : float,
         weight_decay : float,
         l1_strength : float,
+        scheduler,
         activation_type : str = "relu",
         negative_slope : Optional[float] = 0.2,
         n_hidden_units_list : Optional[List[int]] = None,
+        momentum : float=0.9,
         sample_weight : Optional[Union["torch.Tensor", "np.ndarray"]] = None
     ) -> None:
         super(_CustomMLP, self).__init__()
@@ -624,6 +633,8 @@ class _CustomMLP(nn.Module):
         self.l1_strength = l1_strength
         self.activation_type = activation_type
         self.negative_slope = negative_slope
+        self.scheduler = scheduler
+        self.momentum = momentum
 
         for n_hidden_units in self.n_hidden_units_list:
             if n_hidden_units <=0:
@@ -710,7 +721,9 @@ class CustomMLP(BaseNet):
             'lr',
             'weight_decay',
             'sample_weight',
-            'l1_strength'
+            'l1_strength',
+            'scheduler',
+            'momentum'
             ]
         default_parameter_dict = {
             'n_hidden_units_list' : None
@@ -765,8 +778,10 @@ class _VGG(nn.Module):
         lr : float,
         weight_decay : float,
         l1_strength : float,
+        scheduler,
         activation_type : str = "relu",
         negative_slope : Optional[float] = None, 
+        momentum : float=0.9,
         sample_weight : Optional[Union["torch.Tensor", "np.ndarray"]] = None
     ) -> None: 
         
@@ -797,7 +812,9 @@ class _VGG(nn.Module):
         self.l1_strength = l1_strength
         self.activation_type = activation_type
         self.negative_slope = negative_slope
-    
+        self.scheduler = scheduler
+        self.momentum = momentum
+
         self.avgpool = nn.AdaptiveAvgPool2d((7, 7))
         layers = []
         layers.append(nn.Linear(512 * 7 * 7, 4096))
@@ -901,7 +918,9 @@ class VGG(ConvNet):
             'sample_weight',
             'l1_strength',
             'activation_type',
-            'negative_slope'
+            'negative_slope',
+            'scheduler',
+            'momentum'
             ]
 
         default_parameter_dict = {}
@@ -937,7 +956,103 @@ class VGG(ConvNet):
 
     def forward(self, X):
         return self.model.forward(X)
+
+class _ResNet18(nn.Module):
+    def __init__(
+        self,
+        dims : Union[Tuple[int], int],
+        n_classes : int, 
+        batch_size : int,
+        optimizer_type : str,
+        lr : float,
+        weight_decay : float,
+        l1_strength : float,
+        scheduler,
+        momentum : float=0.9,
+        sample_weight : Optional[Union["torch.Tensor", "np.ndarray"]] = None
+    )->None:
+        super(_ResNet18, self).__init__()
+        def create_model():
+            model = models.resnet18(pretrained=False, num_classes=n_classes)
+            model.conv1 = nn.Conv2d(3, 64, kernel_size=(3, 3), stride=(1, 1), padding=(1, 1), bias=False)
+            model.maxpool = nn.Identity()
+            return model
+        self.dims = dims
+        self.n_classes = n_classes
+        self.batch_size = batch_size
+        self.optimizer_type = optimizer_type
+        self.lr = lr
+        self.weight_decay = weight_decay
+        self.l1_strength = l1_strength
+        self.sample_weight = sample_weight
+        self.momentum = momentum
+        self.scheduler = scheduler
+
+        layers = [create_model()]
+        if self.n_classes > 2:
+            layers.append(nn.Softmax(dim=1))
+        else:
+            layers.append(nn.Sigmoid())
+        self.model = nn.Sequential(*layers)
+
+    def forward(self, X):
+        return self.model(X)
+
+class ResNet18(ConvNet):
+    def __init__(
+        self,
+        dims : Union[int, Tuple[int]],
+        n_classes : int,
+        sample_weight : Optional[Union["torch.Tensor", "np.ndarray"]] = None,
+        **hparams
+    )->None:
+        self.dims = dims
+        self.n_classes = n_classes
+        self.sample_weight = sample_weight
+
+        entire_keys = [
+            'dims', 
+            'n_classes', 
+            'optimizer_type', 
+            'batch_size',
+            'lr',
+            'weight_decay',
+            'sample_weight',
+            'l1_strength',
+            'scheduler',
+            'momentum'
+            ]
+        default_parameter_dict = {
+            
+        }
         
+        # set default for model-specific hyperparameters
+        for argument, default in default_parameter_dict.items():
+            if argument not in hparams.keys(): 
+                hparams.update({argument : default})
+        #set default for model-agnostic hyperparameters
+        set_default_parameters(hparams)
+        # save information of **hparams to self.hparams.
+        hparams.pop('dropout_p')
+        hparams.pop('bn')
+        try:
+            hparams.pop('activation_type')
+        except:
+            pass
+        self.save_hyperparameters(*hparams)
+
+        super(ResNet18, self).__init__(
+            dims=dims,
+            n_classes=n_classes,
+            sample_weight=sample_weight,
+            **hparams
+        )
+
+        self.model = _ResNet18(dims=dims, n_classes=n_classes, sample_weight=sample_weight, **hparams)
+
+    def forward(self, X):
+        return self.model(X)
+
 class _UNet(nn.Module):
     """
     Paper: `U-Net: Convolutional Networks for Biomedical Image Segmentation
@@ -964,6 +1079,7 @@ class _UNet(nn.Module):
         bilinear : bool,
         l1_strength : float,
         activation_type : str = "relu",
+        momentum : float=0.9,
         negative_slope : Optional[float] = None,
         model_type : str = "UNet", # don't change this
         dataformats : Optional[str] = "NCHW", # number of samples, channels, height, width
@@ -984,6 +1100,7 @@ class _UNet(nn.Module):
         self.activation_type = activation_type 
         self.negative_slope = negative_slope
         self.dataformats = dataformats
+        self.momentum = momentum
 
         if self.n_layers < 1:
             raise ValueError(f"n_layers = {self.n_layers}, expected: n_layers > 0")
@@ -1043,7 +1160,8 @@ class UNet(SegNet):
             'l1_strength',
             'activation_type',
             'negative_slope',
-            'dataformats'
+            'dataformats',
+            'momentum'
         ]
 
         default_parameter_dict = {
@@ -1101,6 +1219,7 @@ class _PixelCNN(nn.Module):
         dataformats : Optional[str] = "NCHW", # number of samples, channels, height, width
         hidden_channels : int=256,
         n_blocks : int=5,
+        momentum : float=0.9,
         sample_weight : Optional[Union["torch.Tensor", "np.ndarray"]] = None
     ):
         super().__init__()
@@ -1119,6 +1238,7 @@ class _PixelCNN(nn.Module):
         self.negative_slope = negative_slope
         self.dataformats = dataformats
         self.sample_weight = sample_weight
+        self.momentum = momentum
 
         self.blocks = nn.ModuleList([self.conv_block(in_channels) for _ in range(n_blocks)])
 
@@ -1169,7 +1289,8 @@ class PixelCNN(SegNet):
             'l1_strength',
             'activation_type',
             'negative_slope',
-            'dataformats'
+            'dataformats',
+            'momentum'
         ]
 
         default_parameter_dict = {
@@ -1208,6 +1329,7 @@ class PixelCNN(SegNet):
     def forward(self, X):
         return self.model.forward(X)
 
+
 class _Generator(nn.Module):
     def __init__(
         self,
@@ -1220,7 +1342,8 @@ class _Generator(nn.Module):
         hidden_units : List[int] = [128, 512, 1024],
         latent_dim : int = 100,
         activation_type : str = "leaky_relu",
-        negative_slope : Optional[float] = 0.2
+        negative_slope : Optional[float] = 0.2,
+        momentum : float = 0.9
     )->None:
         super(_Generator, self).__init__()
         self.dims = dims
@@ -1233,7 +1356,7 @@ class _Generator(nn.Module):
         self.bn = bn
         self.activation_type = activation_type
         self.negative_slope = negative_slope
-        
+        self.momentum = momentum
         def block(in_units, out_units):
             layers = [nn.Linear(in_units, out_units)]
             if self.bn:
@@ -1273,7 +1396,8 @@ class _Discriminator(nn.Module):
         hidden_units : List[int] = [512, 256],
         sample_weight : Optional[Union["np.ndarray", "torch.Tensor"]] = None,
         activation_type : str = "leaky_relu",
-        negative_slope : Optional[float] = 0.2
+        negative_slope : Optional[float] = 0.2,
+        momentum : float=0.9
     )->None:
         super(_Discriminator, self).__init__()
         self.dims = dims
@@ -1284,6 +1408,7 @@ class _Discriminator(nn.Module):
         self.sample_weight = sample_weight
         self.activation_type = activation_type
         self.negative_slope = negative_slope
+        self.momentum = momentum
 
         def block(in_units, out_units):
             layers = [nn.Linear(in_units, out_units)]
