@@ -619,6 +619,7 @@ class BaseProject:
                 cols = int(math.sqrt(n_data))
                 fig = plt.figure(figsize=(5*rows, 5*cols))
                 idx = 0
+                dims_imshow = self.dims
                 for i in range(1, rows+1):
                     for j in range(1, cols+1):
                         idx += 1
@@ -626,6 +627,8 @@ class BaseProject:
                             img = fig.add_subplot(rows, cols, idx)
                             if len(self.dims) > 2:
                                 dims_imshow = rearrange_dims(self.dims, in_format=dataformats, dataformats="NHWC")
+                                if dims_imshow[-1] == 1:
+                                    dims_imshow = (dims_imshow[0], dims_imshow[1])
                             img.imshow(imgs_to_show[idx-1].reshape(dims_imshow))
                             img.set_title(f'Image # {idx-1}')
                 plt.show()
@@ -646,13 +649,17 @@ class BaseProject:
             cols = int(math.sqrt(n_data))
             fig = plt.figure(figsize=(5*rows, 5*cols))
             idx = 0
+            dims_imshow = self.dims
             for i in range(1, rows+1):
                 for j in range(1, cols+1):
                     idx += 1
                     if idx < n_data + 1:
                         img = fig.add_subplot(rows, cols, idx)
-                        if len(self.dims) > 2:
+                        
+                        if len(self.dims)==3:
                             dims_imshow = rearrange_dims(self.dims, in_format=dataformats, dataformats="NHWC")
+                            if dims_imshow[-1] == 1:
+                                dims_imshow = (dims_imshow[0], dims_imshow[1])
                         img.imshow(imgs_to_show[idx-1].reshape(dims_imshow))
                         img.set_title(f'label : {labels_to_show[idx-1]}')
             plt.show()
