@@ -31,6 +31,8 @@ def split_data(
         tuple of `torch.Tensors`: The train/val/test data. 
 
     """
+    X_train_tensor, X_val_tensor, X_predict_tensor, X_test_tensor = (None, None, None, None)
+    y_train_tensor, y_val_tensor, y_predict_tensor, y_test_tensor = (None, None, None, None)
     if (X_test is None and y_test is not None) or (X_test is not None and y_test is None):
         raise ValueError("X_test and y_test should both be specified as a certain np.array object, or else all be NoneType objects.")
 
@@ -44,14 +46,11 @@ def split_data(
     else:
         X_train, y_train = X, y
 
-    if not isinstance(X_train, torch.Tensor):
-        X_predict_tensor = torch.Tensor(X_train).float()
-    if not isinstance(y_train, torch.Tensor):
-        y_predict_tensor = torch.Tensor(y_train).float()
-    if not isinstance(X_test, torch.Tensor):
-        X_test_tensor = torch.Tensor(X_test).float()
-    if not isinstance(y_test, torch.Tensor):
-        y_test_tensor = torch.Tensor(y_test).float()
+    
+    X_predict_tensor = torch.Tensor(X_train).float() if not isinstance(X_train, torch.Tensor) else X_train.float()
+    y_predict_tensor = torch.Tensor(y_train).float() if not isinstance(y_train, torch.Tensor) else y_train.float()
+    X_test_tensor = torch.Tensor(X_test).float() if not isinstance(X_test, torch.Tensor) else X_test.float()
+    y_test_tensor = torch.Tensor(y_test).float() if not isinstance(y_test, torch.Tensor) else y_test.float()
 
    
     X_train, X_val, y_train, y_val = train_test_split(
@@ -60,14 +59,11 @@ def split_data(
         random_state=0
     )
     
-    if not isinstance(X_train, torch.Tensor):
-        X_train_tensor = torch.Tensor(X_train).float()
-    if not isinstance(y_train, torch.Tensor):
-        y_train_tensor = torch.Tensor(y_train).float()
-    if not isinstance(X_val, torch.Tensor):
-        X_val_tensor = torch.Tensor(X_val).float()
-    if not isinstance(y_val, torch.Tensor):
-        y_val_tensor = torch.Tensor(y_val).float()
+    
+    X_train_tensor = torch.Tensor(X_train).float() if not isinstance(X_train, torch.Tensor) else X_train.float()
+    y_train_tensor = torch.Tensor(y_train).float() if not isinstance(y_train, torch.Tensor) else y_train.float()
+    X_val_tensor = torch.Tensor(X_val).float() if not isinstance(X_val, torch.Tensor) else X_val.float()
+    y_val_tensor = torch.Tensor(y_val).float() if not isinstance(y_val, torch.Tensor) else y_val.float()
 
     return X_train_tensor, y_train_tensor, X_val_tensor, y_val_tensor, X_predict_tensor, y_predict_tensor, X_test_tensor, y_test_tensor 
 
@@ -102,14 +98,30 @@ class DataModule(pl.LightningDataModule):
 
     def setup(self, stage):
         if self.X_val is not None and self.y_val is not None and self.X_test is not None and self.y_test is not None:
-            self.X_train_tensor = torch.Tensor(self.X).float()
-            self.X_val_tensor = torch.Tensor(self.X_val).float()
-            self.X_test_tensor = torch.Tensor(self.X_test).float()
-            self.y_train_tensor = torch.Tensor(self.y).float()
-            self.y_val_tensor = torch.Tensor(self.y_val).float()
-            self.y_test_tensor = torch.Tensor(self.y_test).float()
-            self.X_predict_tensor = torch.Tensor(np.concatenate([self.X, self.X_val])).float()
-            self.y_predict_tensor = torch.Tensor(np.concatenate([self.y, self.y_val])).float()
+            self.X_train_tensor = torch.Tensor(self.X).float() if not isinstance(self.X, torch.Tensor) else self.X.float()
+            self.X_val_tensor = torch.Tensor(self.X_val).float() if not isinstance(self.X_val, torch.Tensor) else self.X_val.float()
+            self.X_test_tensor = torch.Tensor(self.X_test).float() if not isinstance(self.X_test, torch.Tensor) else self.X_test.float()
+            self.y_train_tensor = torch.Tensor(self.y).float() if not isinstance(self.y, torch.Tensor) else self.y.float()
+            self.y_val_tensor = torch.Tensor(self.y_val).float() if not isinstance(self.y_val, torch.Tensor) else self.y_val.float()
+            self.y_test_tensor = torch.Tensor(self.y_test).float() if not isinstance(self.y_test, torch.Tensor) else self.y_test.float()
+            if not isinstance(self.X, torch.Tensor) and not isinstance(self.X_val, torch.Tensor):
+                self.X_predict_tensor = torch.Tensor(np.concatenate([self.X, self.X_val])).float()
+            else:
+                if isinstance(self.X, torch.Tensor):
+                    self.X = self.X.to_numpy().float()
+                if isinstance(self.X_val, torch.Tensor):
+                    self.X_val = self.X_val.to_numpy().float()
+                self.X_predict_tensor = torch.Tensor(np.concatenate([self.X, self.X_val])).float()
+
+            if not isinstance(self.y, torch.Tensor) and not isinstance(self.y_val, torch.Tensor):
+                self.y_predict_tensor = torch.Tensor(np.concatenate([self.y, self.y_val])).float()
+            else:
+                if isinstance(self.y, torch.Tensor):
+                    self.y = self.y.to_numpy().float()
+                if isinstance(self.y_val, torch.Tensor):
+                    self.y_val = self.y_val.to_numpy().float()
+                self.y_predict_tensor = torch.Tensor(np.concatenate([self.y, self.y_val])).float()
+            
         else:
             self.X_train_tensor, self.y_train_tensor, self.X_val_tensor, self.y_val_tensor, self.X_predict_tensor, self.y_predict_tensor, self.X_test_tensor, self.y_test_tensor = split_data(
                 X=self.X, 
